package cz.muni.fi.pb162.cviceni;


/**
 * Implementace rozhrani Edge. 
 * Hrany jsou neorientovane.
 * Hrany jsou porovnavatelne podle delky. 
 *
 * @author Zuzana Neverilova
 * @version 1
 */
public class EdgeImpl implements Edge, Comparable
{
    private Node from, to;
    private int length;

    /**
     * Konstruktor
     * @param from uzel, ze ktereho hrana vychazi
     * @param to uzel, do ktereho hrana smeruje
     * @param length delka hrany
     * @throws NullPointerException, pokud je nektery z uzlu null
     */
    public EdgeImpl(Node from, Node to, int length)
    {
        if (from == null)
            throw new NullPointerException("from");
        if (to == null)
            throw new NullPointerException("to");            
        this.from = from;
        this.to = to;
        this.length = length;
    }
    
    /**
     * Vraci delku hrany
     * @return delka hrany
     */
    public int getLength()
    {
        return length;
    }
    
    /**
     * Vraci uzel, ze ktereho hrana vychazi.
     * @return uzel, ze ktereho hrana vychazi
     */
    public Node getFrom()
    {
        return from;
    }
    
    /**
     * Vraci uzel, do ktereho hrana smeruje.
     * @return uzel, do ktereho hrana smeruje
     */
    public Node getTo()
    {
        return to;
    }
    
    /**
     * Rovnost hran. Hrany jsou stejne, pokud jsou hrany a maji
     * stejne oba uzly.
     */
    public boolean equals(Object o)
    {
        if (o instanceof Edge)
            return (((Edge)o).getFrom().equals(from)&&
                    ((Edge)o).getTo().equals(to)||
                    ((Edge)o).getFrom().equals(to)&& // neorientovany graf
                    ((Edge)o).getTo().equals(from));
        return false;
    }
    /**
     * Hashcode hrany je absolutni hodnota rozdilu obou hashcodu.
     * Takze hrana z A do B ma stejny hashCode jako hrana z B do A.
     */
    public int hashCode()
    {
        return Math.abs(from.hashCode() - to.hashCode());
    }
            
    /**
     * Porovnavani hran. Pokud maji hrany stejnou delku, porovnava se podle
     * jmen uzlu.
     * @param o objekt k porovnani
     * @return porovnani hran
     */
    public int compareTo(Object o)
    {
        if (o instanceof Edge)
        {
            Edge e = (Edge)o;
            if (e.getLength() == length)
                return (e.getFrom().getName()+e.getTo().getName())
                    .compareTo(from.getName() + to.getName());
            return length-e.getLength();
        } else 
        {
            throw new ClassCastException(o+" neni typu Edge");
        }
    }
    
    /**
     * Textova reprezentace hrany
     * @return textova reprezentace hrany
     */
    public String toString()
    {
        return "Hrana z "+from+" do "+to+" ("+length+")";
    }
}
